#include <stdio.h>
#include <libxml/xmlreader.h>
#include <string.h>
#include <libharkio3.h>

enum checkType {SRC,TSP,IMPULSE,MIC,NOISE,UNKNOWN};
int validate_harkxml(harkio_XML* xml);
void help(void);




/*--------------------------------------------------*/
//
//help
/*--------------------------------------------------*/
void help(void)
{
    printf("                                                     \n");
    printf("                                                     \n");
    printf("[NAME:]                                              \n");
    printf("  harktoolcli-validxml                               \n");
    printf("                                                     \n");
    printf("[Description:]                                       \n");
    printf("  Consistency check of xml file                      \n");
    printf("                                                     \n");
    printf("[Usage :]                                            \n");
    printf("  harktoolcli-validxml filename(xml)                 \n");
    printf("                                                     \n");
    printf("    positions_type:                                  \n");
    printf("      microphone,source,tsp,impulse,noise            \n");
    printf("                                                     \n");
    printf("[Output :]                                           \n");
    printf("  If a xml_file is valid, the command retruns 0.     \n");
    printf("                                                     \n");
    printf("                                                     \n");
}




/*--------------------------------------------------*/
//
//
/*--------------------------------------------------*/
int main(int argc, char **argv)
{
    int ret;

    // argc check
    if (argc <= 1) {
        harkio_Log_printf(harkio_Log_Error , "argument error!!");
        help();
        exit(EXIT_FAILURE);
    }

    // help
    if (strcmp(argv[1] , "--help") == 0) {
        help();
        exit(EXIT_SUCCESS);
    }

    harkio_XML* io3xml = harkio_XML_newFromXML(argv[1], "");
    ret = validate_harkxml(io3xml);
    return (ret);
}




/*---------------------------------------------------------*/
//
//
/*---------------------------------------------------------*/
int validate_harkxml(harkio_XML* xml)
{

    char msg[1024];
    memset(msg ,0 , 1024);
    enum checkType poses_case;


    if (xml == NULL) {
        strcat(msg, "[hark_xml]");
    } else {
        if (!xml->poses) {
            strcat(msg, "[positions]");
        } else {
            char* poses_type_str = xml->poses->type ? xml->poses->type : "";
            if (strcmp(poses_type_str ,  "source") == 0) {
                poses_case = SRC;
            } else if (strcmp(poses_type_str ,  "tsp") == 0) {
                poses_case = TSP;
            } else if (strcmp(poses_type_str ,  "impulse") == 0) {
                poses_case = IMPULSE;
            } else if (strcmp(poses_type_str ,  "noise") == 0) {
                poses_case = NOISE;
            } else if (strcmp(poses_type_str ,  "microphone") == 0) {
                poses_case = MIC;
            } else {
                poses_case = UNKNOWN;
                // printf("Consistency error: Unknown positions!! [%s]\n" , xml->poses->type );
                strcat(msg, "[positions (unknown type: ");
                strcat(msg, poses_type_str);
                strcat(msg, ")]");
            }

            switch (poses_case) {
            case SRC:
            case TSP:
            case IMPULSE:

                if (xml->cfg == NULL) {
                    strcat(msg ,  "[config]");
                } else {
                    if (xml->cfg->comment == NULL) {
                        strcat(msg , "[comment]");
                    }
                    if (xml->cfg->synchronousAverage < 0) {
                        strcat(msg , "[SynchronousAverage]");
                    }
                    if (xml->cfg->TSPpath      == NULL) {
                        strcat(msg , "[TSPpath]");
                    }
                    if (xml->cfg->TSPoffset    < 0) {
                        strcat(msg , "[TSPOffset]");
                    }
                    if (xml->cfg->peakSearchFrom < 0 || xml->cfg->peakSearchTo < 0) {
                        strcat(msg , "[PeakSearch]");
                    }
                    if (xml->cfg->nfft         <= 0) {
                        strcat(msg , "[nfft]");
                    }
                    if (xml->cfg->samplingRate <= 0) {
                        strcat(msg , "[samplingRate]");
                    }
                    if (xml->cfg->signalMax    <= 0) {
                        strcat(msg , "[signalMax]");
                    }
                    if (xml->cfg->TSPLength    <= 0) {
                        strcat(msg , "[TSPLength]");
                    }
                }

                // do not default
            case NOISE:
            case MIC:
            default:
                //positions tag
                if (xml->poses && xml->poses->size <= 0) {
                    strcat(msg , "[position]");
                }
                //neighbors tag
                if (xml->nbrs    == NULL) {
                    strcat(msg , "[neighbors]");
                } else {
                    if (xml->nbrs->size <= 0) {
                        strcat(msg , "[neighbor]");
                    }
                }
            }
        }
    }

    if (strlen(msg) == 0) {
        fprintf(stderr, "SUCCESS\n");
        return (EXIT_SUCCESS);
    } else {
        fprintf(stderr, "Consistency error\n");
        fprintf(stderr, "%s\n" , msg);
        return (EXIT_FAILURE);
    }

}
